@@ -5,10 +5,11 @@ from django.contrib.markup.templatetags import markup
 from django.contrib.sites.models import Site
 from django.core.cache import cache
 from django.conf import settings
-from django.template.defaultfilters import striptags
+from django.template.defaultfilters import slugify, striptags
 from django.utils.translation import ugettext_lazy as _
 from datetime import datetime
 from base64 import encodestring
+import mimetypes
 import re
 import urllib
 
@@ -83,6 +84,32 @@ class Tag(models.Model):
     class Meta:
         ordering = ('name',)
 
+class ArticleStatusManager(models.Manager):
+    def default(self):
+        default = self.all()[:1]
+
+        if len(default) == 0:
+            return None
+        else:
+            return default[0]
+
+class ArticleStatus(models.Model):
+    name = models.CharField(max_length=50)
+    ordering = models.IntegerField(default=0)
+    is_live = models.BooleanField(default=False, blank=True)
+
+    objects = ArticleStatusManager()
+
+    class Meta:
+        ordering = ('ordering', 'name')
+        verbose_name_plural = _('Article statuses')
+
+    def __unicode__(self):
+        if self.is_live:
+            return u'%s (live)' % self.name
+        else:
+            return self.name
+
 class ArticleManager(models.Manager):
     def active(self):
         """
@@ -96,6 +123,18 @@ class ArticleManager(models.Manager):
                 publish_date__lte=now,
                 is_active=True)
 
+    def live(self, user=None):
+        """Retrieves all live articles"""
+
+        qs = self.active()
+
+        if user is not None and user.is_superuser:
+            # superusers get to see all articles
+            return qs
+        else:
+            # only show live articles to regular users
+            return qs.filter(status__is_live=True)
+
 MARKUP_HELP = _("""Select the type of markup you are using in this article.
 <ul>
 <li><a href="http://daringfireball.net/projects/markdown/basics" target="_blank">Markdown Guide</a></li>
@@ -106,6 +145,7 @@ MARKUP_HELP = _("""Select the type of markup you are using in this article.
 class Article(models.Model):
     title = models.CharField(max_length=100)
     slug = models.SlugField(unique_for_year='publish_date')
+    status = models.ForeignKey(ArticleStatus, default=ArticleStatus.objects.default)
     author = models.ForeignKey(User)
     sites = models.ManyToManyField(Site, blank=True)
 
@@ -174,6 +214,14 @@ class Article(models.Model):
         if self.use_addthis_button and self.addthis_use_author and not self.addthis_username:
             self.addthis_username = self.author.username
 
+        # make sure the slug is always unique for the year this article was posted
+        if not self.id:
+            # make sure we have a slug first
+            if not len(self.slug.strip()):
+                self.slug = slugify(self.title)
+
+            self.slug = self.get_unique_slug(self.slug)
+
         super(Article, self).save(*args, **kwargs)
         requires_save = False
 
@@ -195,6 +243,25 @@ class Article(models.Model):
         if requires_save:
             super(Article, self).save(*args, **kwargs)
 
+    def get_unique_slug(self, slug):
+        """Iterates until a unique slug is found"""
+
+        # we need a publish date before we can do anything meaningful
+        if type(self.publish_date) is not datetime:
+            return slug
+
+        orig_slug = slug
+        year = self.publish_date.year
+        counter = 1
+
+        while True:
+            not_unique = Article.objects.filter(publish_date__year=year, slug=slug)
+            if len(not_unique) == 0:
+                return slug
+
+            slug = '%s-%s' % (orig_slug, counter)
+            counter += 1
+
     def _get_article_links(self):
         """
         Find all links in this article.  When a link is encountered in the
@@ -278,11 +345,11 @@ class Article(models.Model):
     teaser = property(_get_teaser)
 
     def get_next_article(self):
-        """Determines the next active article"""
+        """Determines the next live article"""
 
         if not self._next:
             try:
-                qs = Article.objects.active().exclude(id__exact=self.id)
+                qs = Article.objects.live().exclude(id__exact=self.id)
                 article = qs.filter(publish_date__gte=self.publish_date).order_by('publish_date')[0]
             except (Article.DoesNotExist, IndexError):
                 article = None
@@ -291,11 +358,11 @@ class Article(models.Model):
         return self._next
 
     def get_previous_article(self):
-        """Determines the previous active article"""
+        """Determines the previous live article"""
 
         if not self._previous:
             try:
-                qs = Article.objects.active().exclude(id__exact=self.id)
+                qs = Article.objects.live().exclude(id__exact=self.id)
                 article = qs.filter(publish_date__lte=self.publish_date).order_by('-publish_date')[0]
             except (Article.DoesNotExist, IndexError):
                 article = None
@@ -306,3 +373,31 @@ class Article(models.Model):
     class Meta:
         ordering = ('-publish_date', 'title')
 
+class Attachment(models.Model):
+    upload_to = lambda inst, fn: 'attach/%s/%s/%s' % (datetime.now().year, inst.article.slug, fn)
+
+    article = models.ForeignKey(Article, related_name='attachments')
+    attachment = models.FileField(upload_to=upload_to)
+    caption = models.CharField(max_length=255, blank=True)
+
+    class Meta:
+        ordering = ('-article', 'id')
+
+    def __unicode__(self):
+        return u'%s: %s' % (self.article, self.caption)
+
+    @property
+    def filename(self):
+        return self.attachment.name.split('/')[-1]
+
+    @property
+    def content_type_class(self):
+        mt = mimetypes.guess_type(self.attachment.path)[0]
+        if mt:
+            content_type = mt.replace('/', '_')
+        else:
+            # assume everything else is text/plain
+            content_type = 'text_plain'
+
+        return content_type
+
